from metasim.cfg.objects import ArticulationObjMetaCfg, RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class LampOffMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/lamp_off/v2"
    objects = [
        RigidObjMetaCfg(
            name="lamp_base",
            filepath="roboverse_data/assets/rlbench/lamp_off/lamp_base/usd/lamp_base.usd",
            physics=PhysicStateType.GEOM,
        ),
        ArticulationObjMetaCfg(
            name="push_button_target",
            filepath="roboverse_data/assets/rlbench/lamp_off/push_button_target/usd/push_button_target.usd",
        ),
    ]
    # TODO: add checker



from __future__ import annotations

from metasim.utils.configclass import configclass


@configclass
class SceneMetaCfg:
    """Base config class for scenes"""

    name: str | None = None
    filepath: str | None = None
    urdf_path: str | None = None
    mjcf_path: str | None = None

    positions: list[tuple[float, float, float]] | None = None
    default_position: tuple[float, float, float] | None = None
    quat: tuple[float, float, float, float] | None = None

    scale: tuple[float, float, float] | None = None



# Copyright (c) 2022-2024, The Isaac Lab Project Developers.
# All rights reserved.
#
# SPDX-License-Identifier: BSD-3-Clause

"""Sub-module containing utilities for transforming strings and regular expressions."""

import ast
import importlib
import inspect
import re
from collections.abc import Callable
from dataclasses import is_dataclass

"""
String formatting.
"""


def is_camel_case(name: str) -> bool:
    # see https://stackoverflow.com/a/10182711
    return re.match(r"^(?:[A-Z0-9][a-z]*)+$", name) is not None


def is_snake_case(name: str) -> bool:
    return re.match(r"^[a-z][a-z0-9]*(_[a-z0-9]+)*$", name) is not None


def to_camel_case(snake_str: str, to: str = "CC") -> str:
    components = snake_str.lower().split("_")
    return "".join(x.title() for x in components)


def to_snake_case(camel_str: str) -> str:
    """Converts a string from camel case to snake case.

    Args:
        camel_str: A string in camel case.

    Returns:
        A string in snake case (i.e. with '_')
    """
    camel_str = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", camel_str)
    return re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", camel_str).lower()


"""
String <-> Callable operations.
"""


def is_lambda_expression(name: str) -> bool:
    """Checks if the input string is a lambda expression.

    Args:
        name: The input string.

    Returns:
        Whether the input string is a lambda expression.
    """
    try:
        ast.parse(name)
        return isinstance(ast.parse(name).body[0], ast.Expr) and isinstance(ast.parse(name).body[0].value, ast.Lambda)
    except SyntaxError:
        return False


def callable_to_string(value: Callable) -> str:
    """Converts a callable object to a string.

    Args:
        value: A callable object.

    Raises:
        ValueError: When the input argument is not a callable object.

    Returns:
        A string representation of the callable object.
    """
    # check if callable
    if not callable(value):
        raise ValueError(f"The input argument is not callable: {value}.")
    # check if lambda function
    if value.__name__ == "<lambda>":
        # we resolve the lambda expression by checking the source code and extracting the line with lambda expression
        # we also remove any comments from the line
        lambda_line = inspect.getsourcelines(value)[0][0].strip().split("lambda")[1].strip().split(",")[0]
        lambda_line = re.sub(r"#.*$", "", lambda_line).rstrip()
        return f"lambda {lambda_line}"
    else:
        # get the module and function name
        module_name = value.__module__
        function_name = value.__name__
        # return the string
        return f"{module_name}:{function_name}"


def string_to_callable(name: str) -> Callable:
    """Resolves the module and function names to return the function.

    Args:
        name: The function name. The format should be 'module:attribute_name' or a
            lambda expression of format: 'lambda x: x'.

    Raises:
        ValueError: When the resolved attribute is not a function.
        ValueError: When the module cannot be found.

    Returns:
        Callable: The function loaded from the module.
    """
    try:
        if is_lambda_expression(name):
            callable_object = eval(name)
        else:
            mod_name, attr_name = name.split(":")
            mod = importlib.import_module(mod_name)
            callable_object = getattr(mod, attr_name)
        # check if attribute is callable
        if callable(callable_object):
            return callable_object
        else:
            raise AttributeError(f"The imported object is not callable: '{name}'")
    except (ValueError, ModuleNotFoundError) as e:
        msg = (
            f"Could not resolve the input string '{name}' into callable object."
            " The format of input should be 'module:attribute_name'.\n"
            f"Received the error:\n {e}."
        )
        raise ValueError(msg) from e


def string_to_dataclass_instance(name: str) -> Callable:
    try:
        mod_name, attr_name = name.split(":")
        mod = importlib.import_module(mod_name)
        dataclass_instance = getattr(mod, attr_name)
        if is_dataclass(dataclass_instance):
            if isinstance(dataclass_instance, type):
                return dataclass_instance()
            else:
                return dataclass_instance
        else:
            raise AttributeError(f"The imported object is not a dataclass: '{name}'")
    except (ValueError, ModuleNotFoundError) as e:
        msg = (
            f"Could not resolve the input string '{name}' into dataclass instance."
            " The format of input should be 'module:attribute_name'.\n"
            f"Received the error:\n {e}."
        )
        raise ValueError(msg) from e



import os

import omni
import omni.isaac.core.utils.prims as prim_utils
from omni.kit.material.library import get_material_prim_path
from pxr import UsdShade


def apply_mdl_to_prim(mdl_path: str, prim_path: str):
    assert os.path.exists(mdl_path), f"Material file {mdl_path} does not exist"
    assert mdl_path.endswith(".mdl"), f"Material file {mdl_path} must have .mdl extension"
    prim = prim_utils.get_prim_at_path(prim_path)

    mtl_name = os.path.basename(mdl_path).removesuffix(".mdl")
    _, mtl_prim_path = get_material_prim_path(mtl_name)
    success, result = omni.kit.commands.execute(
        "CreateMdlMaterialPrim",
        mtl_url=mdl_path,
        mtl_name=mtl_name,
        mtl_path=mtl_prim_path,
        select_new_prim=False,
    )
    if not success:
        raise RuntimeError(f"Failed to create material {mtl_name} at {mtl_prim_path}")

    success, result = omni.kit.commands.execute(
        "BindMaterial",
        prim_path=prim.GetPath(),
        material_path=mtl_prim_path,
        strength=UsdShade.Tokens.strongerThanDescendants,
    )
    if not success:
        raise RuntimeError(f"Failed to bind material at {mtl_prim_path} to {prim.GetPath()}")



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotHandleRightJawLeftCalipers324MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-213733_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotHandleRightJawLeftCalipers325MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-213410_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotHandleRightJawLeftCalipers326MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-213404_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotHandleRightJawLeftCalipers327MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-215200_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotHandleRightJawLeftCalipers328MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-213858_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotHandleRightJawLeftCalipers329MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="calipers",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/95c967114b624291bc26bcd705aaa334/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/95c967114b624291bc26bcd705aaa334_handle_right_jaw_left/20240826-211405_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from metasim.cfg.tasks import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, TaskType
from metasim.utils import configclass


@configclass
class GAPartNetTaskMetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.GAPARTNET
    task_type = TaskType.TABLETOP_MANIPULATION
    can_tabletop = True
